c     Copyright (C)
c     2010-2013 Universit Bordeaux 1
c     2010-2013 Ecole des Mines d'Albi
c     2014-2018 - Mesostar
c     
c     This file must be used under the terms of the CeCILL license.
c     This source file is licensed as described in the file COPYING, which
c     you should have received as part of this distribution.  The terms
c     are also available at
c     http://www.cecill.info/licences/Licence_CeCILL_V2.1-en.txt
c     
      subroutine discrete_contribution_function(Nlay,Nb,
     &     first_band,last_band,
     &     z,
     &     band_is_NER,
     &     band_net_flux_toa,
     &     discrete_cf)
      implicit none
      include 'max.inc'
c
c     Purpose: to compute the contribution function in each narrowband
c     The contribution function is defined as the fraction of energy that leaves the
c     atmosphere as a function of its emission altitude.
c     
c     Input:
c       + Nlay: number of atmospheric layers
c       + Nb: number of spectral intervals
c       + first_band: index of the first integration narrowband
c       + last_band: index of the last integration narrowband
c       + z: altitude at the top of each layer
c       + band_is_NER: NER between each layer and space, in each narrowband
c       + band_net_flux_toa: net flux @ TOA, in each narrowband
c
c     Output:
c       + discrete_cf: contribution function, as a function of the emission layer (discrete)
c
c     I/O
      integer Nlay,Nb
      integer first_band,last_band
      double precision z(0:Nlay_mx)
      double precision band_is_NER(0:Nlay_mx,1:Nb_mx)
      double precision band_net_flux_toa(1:Nb_mx)
      double precision discrete_cf(0:Nlay_mx,1:Nb_mx)
c     temp
      integer i,band
c     parameters
      double precision epsilon
      parameter(epsilon=1.0D-10)
c     label
      integer strlen
      character*(Nchar_mx) label
      label='subroutine discrete_contribution_function'

      do band=first_band,last_band
         do i=0,Nlay
            if ((band_is_NER(i,band).lt.0.0D+0).and.
     &           (dabs(band_is_NER(i,band)).gt.epsilon))then
               call warning(label)
               write(*,*) 'band_is_NER(',i,',',band,')=',
     &              band_is_NER(i,band)
               write(*,*) 'is not supposed to be negative'
            endif
c     The contribution function is the Net Exchange between each layer (ground included)
c     and space, normalized by the total (the net flux @ TOA in the current band).
            discrete_cf(i,band)=band_is_NER(i,band)
     &           /band_net_flux_toa(band)
         enddo ! i
      enddo ! band

      return
      end
